<?php
namespace FiloBlu\Flow\Model\Parser;

abstract class AbstractParser{
    public $map;
    private $attribute_set_id;
    private $configurable_attributes;
    private $websites;
    private $stores;
    protected $_channel;

    abstract public function process($resource);

    /**
    * @var Monolog\Logger
    */
    protected $_logger;

    public function __construct(
             \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
    ) {
        $this->_logger = $loggerProvider->getLogger();

    }

    public function setConfig($config)
    {

        return $this;

    }

    public function setChannel($channel) {
        $this->_channel = $channel;
    }

    public function getChannel() {
        return $this->_channel;
    }

    /*
     * Validate single field applying Validators
     */
    protected function _validate($value,$validators,$field){
        if($validators && count($validators)){
            foreach($validators as $validator){
                $validatorClass = 'FiloBlu\Bouncer\Model\Parser\Validators' . "\\" . ucfirst($validator);
                $validator = new $validatorClass();
                if(!$validator->isValid($value)){
                   return false;
                }
            }
        }
        return true;
    }
    /*
     * Validate single field applying Validators
     */
    protected function _filter($value,$filters,$data){
        if($filters && count($filters) > 0) {
            foreach ($filters as $filterData) {
                $filterClass = 'FiloBlu\Flow\Model\Filters' . "\\" . ucfirst($filterData['class']);
                $filter =  \Magento\Framework\App\ObjectManager::getInstance()->create($filterClass);
                if(method_exists($filter,"setRow")) {
                    $filter->setRow($data);
                }
                $value = $filter->filter($value,$filterData['args']);
            }
        }
        return $value;
    }

    public function loadMap($maplocation){
        $map =  \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Map');
        $map->load($maplocation);
        if($map->initialized){
            $this->map = $map;
            return true;
        }
        return false;
    }

    /*
     * Parse Data
     *
     */
    public function parse($data,$key_mode = false){

        // $this->_logger->info('FiloBlu\Flow\Model\Parser\AbstractParser::parse(data = "' . print_r($data, true) . '", $key_mode = ' . $key_mode . ')');
        if($this->check_mapping_count && !$key_mode && !$this->map->checkData($data)){
            throw new \Exception('Data don\'t match mapping count ' . count($data));
        }

        $channel = $this->getChannel();

        $fields = array();
        if($this->map){
            $fields = $this->map->getFields();
        }

        $parsed_data = array();

        // Default behaviour
        if ($this->check_mapping_count && count($fields)) {
            foreach($fields as $i => $field){
                $validators = array();
                $filters = array();
                $fieldMap = $this->map->get($field);
                $key = ($key_mode) ? (isset($fieldMap['from'])?$fieldMap['from']:$i) : $i;
                if(array_key_exists(trim($key), $data)) {
                    $fieldData = $data[trim($key)];
                    $result = $this->executeMappingValidatorsAndFilters($field,$data,$fieldMap,$fieldData);
                    if ($result){
                        $parsed_data[$field] = $result;
                    }
                }
            }
        // This is no check_mapping_count (Talend parse) then CSV headers will match Magento attributes
        } elseif(!$this->check_mapping_count) {
            
            // Merging headers with current data from the csv
            // array_combine gives a warning if arrays have not equal columns number (using foreach)
            foreach ($this->header as $header_key => $header_column_name) {
                $parsed_data[$header_column_name] = $data[$header_key];
                $fieldMap = array();
                if (count($fields)){
                    $fieldMap = $this->map->get($header_column_name);
                }
                $field = $header_column_name;
                $fieldData = $data[$header_key];
                $result = $this->executeMappingValidatorsAndFilters($field,$data,$fieldMap,$fieldData);
                if ($result){
                    $parsed_data[$field] = $result;
                }
            }
        }

        if (method_exists($channel, 'insertData') && !is_null($channel)) {

            try {
                
                // $this->_logger->info('FiloBlu\Flow\Model\Parser\AbstractParser::parse, parsed_data = "' . print_r($parsed_data, true));
                $processed = $channel->insertData($parsed_data);
            }
            catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            }

        }
        else
            throw new \Exception('no method insertData for class ' . get_class($channel));


    }


    private function executeMappingValidatorsAndFilters($field,$data,$fieldMap,$fieldData){
        if($fieldMap && !isset($fieldMap['skip'])) {
            $validators = null;
            if (isset($fieldMap['validators'])) {
                $validators = $fieldMap['validators'];
            }
            $filters = null;
            if (isset($fieldMap['filters'])) {
                $filters = $fieldMap['filters'];
            }
            if ($this->_validate($fieldData, $validators, $field)) {
                $value = $this->_filter($fieldData, $filters, $data);
                return $value;
            } else {
                throw new \Exception("{$value} is not Valid for {$field}");
            }
        }
        return null;
    }


}
